import nltk
from nltk.corpus import wordnet
import inflect
import itertools

def run_query_syntax(args):

    if 'init' == args.mode:
        nltk.download('wordnet')
        nltk.download('averaged_perceptron_tagger')
        print('Successfully downloaded wordnet and other thesaurus!')
    elif 'run' == args.mode:

        p = inflect.engine()

        def get_word_variants(word):
            variants = {word, word.lower(), word.upper(), word.capitalize()}
            plural = p.plural(word)
            if plural:
                variants.add(plural)
            singular = p.singular_noun(word)
            if singular:
                variants.add(singular)
            return variants

        def get_word_forms(word):
            word_forms = {}
            for synset in wordnet.synsets(word):
                word_type = synset.pos()
                lemmas = [lemma.name().replace('_', ' ') for lemma in synset.lemmas()]
                if word_type not in word_forms:
                    word_forms[word_type] = set(lemmas)
                else:
                    word_forms[word_type].update(lemmas)
            return word_forms

        def build_search_query(word):
            variants = get_word_variants(word)
            word_forms = get_word_forms(word)
            all_forms = set()
            for forms in word_forms.values():
                for form in forms:
                    all_forms.update(get_word_variants(form))
            all_search_terms = variants.union(all_forms)
            # print(all_search_terms)

            return list(all_search_terms)


        keywordsFilePointer = open(args.inputFilePath, 'r', encoding='utf-8')
        resultFilePointer = open(args.outputFilePath, 'w', encoding='utf-8')
        originalKeywordsList = []
        for line in keywordsFilePointer:
            keyword = line.strip('\n')
            # word = "enzyme"
            all_search_terms_list = build_search_query(keyword)
            originalKeywordsList.append(all_search_terms_list)
            resultFilePointer.write(keyword + ':\n' + ' | '.join(all_search_terms_list) + '\n')

        resultFilePointer.write('Note: Please determine the most professional keywords that you need. For example, '
                                  'through professional thesaurus such as MeSH, use * to reduce the possible repetition of words.\n')

        #  originalKeywordsList = [['chen', 'hui'], ['xu', 'gang'], ['long', 'yang']]

        combinations = list(itertools.product(*originalKeywordsList))


        # pubmed
        def format_combination(combination):
            formatted = f"({combination[0]}[Title/Abstract])"
            for word in combination[1:]:
                formatted = f"({formatted} AND ({word}[Title/Abstract]))"
            return formatted

        formatted_strings = [format_combination(combination) for combination in combinations]

        resultFilePointer.write('\nPubMed database search terms are as follows:\n')

        for string in formatted_strings:
            resultFilePointer.write(string[1:-1]+'\n')
            # print(string[1:-1])
        resultFilePointer.write(
            'You can simply copy the query and paste it to the PubMed website (https://pubmed.ncbi.nlm.nih.gov/advanced/) for retrieval.\n')

        def transformPubmedLink(string):
            replacements = {
                '[': '%5B',
                '/': '%2F',
                ']': '%5D',
                '(': '%28',
                ')': '%29',
                ' AND ': '+AND+'
            }
            for old, new in replacements.items():
                string = string.replace(old, new)

            pubmedWebsiteString = 'https://pubmed.ncbi.nlm.nih.gov/?term=' + string + '&sort='
            return pubmedWebsiteString

        resultFilePointer.write('\nHere, we automatically generate the corresponding pubmed search results URL link for you:\n')
        for string in formatted_strings:
            resultFilePointer.write(transformPubmedLink(string[1:-1])+'\n')


        # web of science
        def format_combination2(combination):
            formatted = f"(ALL=({combination[0]}))"
            for word in combination[1:]:
                formatted = f"({formatted} AND ALL=({word}))"
            return formatted

        formatted_strings = [format_combination2(combination) for combination in combinations]

        resultFilePointer.write('\nWeb of Science database search terms are as follows:\n')

        for string in formatted_strings:
            resultFilePointer.write(string[1:-1] + '\n')

        resultFilePointer.write(
            'You can simply copy the query and paste it to the Web of Science website (https://www.webofscience.com/wos/woscc/advanced-search) for retrieval.\n')

        keywordsFilePointer.close()
        resultFilePointer.close()

    # Created by Huilong Chen.



import pandas as pd

def run_remove_duplicates(args):

    file_path = args.inputFilePath

    df = pd.read_excel(file_path)

    df = df.dropna(subset=['DOI'])

    df = df.drop_duplicates(subset=['DOI'])

    output_file = args.outputFilePath

    df.to_csv(output_file, index=False, sep='\t', encoding='utf-8')

    # Made by Huilong Chen.




import argparse
from ptol import query_syntax, remove_duplicates, download_pdfs, pdf_to_text

def main():
    parser = argparse.ArgumentParser(description="ptol: A pipeline for obtaining relevant literature based on given keywords")
    subparsers = parser.add_subparsers(dest='command')

    # query_syntax 子命令
    parser_qs = subparsers.add_parser('query_syntax', help='Generate query statements for literature database searches')
    parser_qs.add_argument('-m', '--mode', choices=['init', 'run'], required=True, help='Mode: init for initialization, run for normal operation')
    parser_qs.add_argument('-i', '--inputFilePath', required=True, help='Input file path containing keywords')
    parser_qs.add_argument('-o', '--outputFilePath', required=True, help='Output file path for query statements')
    parser_qs.set_defaults(func=query_syntax.run_query_syntax)

    # 为其他脚本重复以上步骤来添加子命令
    # remove_duplicates 子命令
    parser_rd = subparsers.add_parser('remove_duplicates', help='Remove duplicate literature')
    parser_rd.add_argument('-i', '--inputFilePath', required=True, help='Input the path to an excel file with a .xlsx extension')
    parser_rd.add_argument('-o', '--outputFilePath', required=True, help='Output file path')
    parser_rd.set_defaults(func=remove_duplicates.run_remove_duplicates)
    
    # download_pdfs 子命令
    parser_dp = subparsers.add_parser('download_pdfs', help='Batch download pdf from sci-hub database according to DOI number')
    parser_dp.add_argument('-i', '--inputFilePath', required=True,
                           help='Input the path to an text file')
    parser_dp.add_argument('-c', '--inputChromedriverPath', required=True, help='Specify the path to chromedriver.exe')
    parser_dp.set_defaults(func=download_pdfs.run_download_pdfs)

    # pdf_to_text 子命令
    parser_pt = subparsers.add_parser('pdf_to_text', help='Batch pdf to text')
    parser_pt.add_argument('-m', '--method', choices=['1', '2', '3', '4'], required=True,
                           help='Input pdf to text method number, there are 1, 2, 3, 4 optional, can only set one of them')
    parser_pt.add_argument('-i', '--inputFolderPath', required=True, help='Enter the path to the folder that contains only pdf literatures')
    parser_pt.add_argument('-o', '--outputFolderPath', required=True, help='Output folder path')
    parser_pt.set_defaults(func=pdf_to_text.run_pdf_to_text)



    args = parser.parse_args()

    if hasattr(args, 'func'):
        args.func(args)
    else:
        parser.print_help()

if __name__ == '__main__':
    main()



import os

def run_pdf_to_text(args):
    folder1 = args.inputFolderPath
    dirList1 = os.listdir(folder1)
    dirPath1 = os.path.abspath(folder1)

    folder2 = args.outputFolderPath
    def mkdir(path):
        folder = os.path.exists(path)
        if not folder:
            os.makedirs(path)
        else:
            pass
    mkdir(folder2)
    dirPath2 = os.path.abspath(folder2)

    if '1' == args.method:
        import PyPDF2
    elif '2' == args.method:
        import pdfplumber
    elif '3' == args.method:
        import fitz
    elif '4' == args.method:
        # from pdfminer.high_level import extract_text
        import io
        from pdfminer.converter import TextConverter
        # from pdfminer.pdfdocument import PDFTextExtractionNotAllowed
        from pdfminer.layout import LAParams
        from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
        from pdfminer.pdfpage import PDFPage

    def pdfToTextMethod1(filePath):
        with open(filePath, 'rb') as file:
            reader = PyPDF2.PdfReader(file)
            text = ''
            for page in reader.pages:
                text += page.extract_text()
        return text

    def pdfToTextMethod2(filePath):
        with pdfplumber.open(filePath) as pdf:
            text = ''
            for page in pdf.pages:
                text += page.extract_text()
            return text

    def pdfToTextMethod3(filePath):
        # with fitz.open(filePath) as pdf:
        #     text = ''
        #     for page in pdf.pages():
        #         text += page.get_text()
        #     return text
        document = fitz.open(filePath)
        text = ""
        for page in document:
            page_text = page.get_text()
            text += page_text
        document.close()

    def pdfToTextMethod4(filePath):
        # text = extract_text(filePath)
        # return text
        resource_manager = PDFResourceManager()
        fake_file_handle = io.StringIO()
        laparams = LAParams(line_overlap=0.5, char_margin=2.0, line_margin=0.5, word_margin=0.1, boxes_flow=0.5,
                            detect_vertical=False, all_texts=False)
        converter = TextConverter(resource_manager, fake_file_handle, laparams=laparams)
        page_interpreter = PDFPageInterpreter(resource_manager, converter)
        with open(filePath, 'rb') as fh:
            for page in PDFPage.get_pages(fh, caching=True, check_extractable=True):
                page_interpreter.process_page(page)
            text = fake_file_handle.getvalue()
        converter.close()
        fake_file_handle.close()
        return text


    for fileName1 in dirList1:
        filePath1 = os.path.join(dirPath1, fileName1)
        textResultPathName = dirPath2 + '/' + fileName1 + '.txt'
        try:
            if '1' == args.method:
                pdfText = pdfToTextMethod1(filePath1)
            elif '2' == args.method:
                pdfText = pdfToTextMethod2(filePath1)
            elif '3' == args.method:
                pdfText = pdfToTextMethod3(filePath1)
            elif '4' == args.method:
                pdfText = pdfToTextMethod4(filePath1)

            outFile = open(textResultPathName, 'w', encoding='utf-8')
            outFile.write(pdfText)
            outFile.close()
        except BaseException as e:
            print('failed:', fileName1, '|',  e)

    # Created by Huilong Chen.



import pandas as pd
from selenium import webdriver
import time
import os
import random

def run_download_pdfs(args):
    # chrome_driver_path = "D:/Chrome Downloads/chromedriver.exe"
    chrome_driver_path = args.inputChromedriverPath

    # file_path = 'all_database_literatures_data_single.txt'
    file_path = args.inputFilePath

    table = pd.read_csv(file_path, sep='\t', header=None)

    titles = table[0].copy(deep=True)
    dois = table[1].copy(deep=True)

    def clean_filename(filename):
        invalid_chars = ['<', '>', ':', '"', '/', '\\', '|', '?', '*']
        for ch in invalid_chars:
            filename = filename.replace(ch, '_')
        return filename

    def scihub_get(doi, download_path):
        chromeOptions = webdriver.ChromeOptions()
        prefs = {"download.default_directory": download_path}
        chromeOptions.add_experimental_option("prefs", prefs)
        wd = webdriver.Chrome(executable_path=chrome_driver_path, options=chromeOptions)

        scihub = ['https://sci-hub.ru/', 'https://sci-hub.st/', 'https://sci-hub.se/']
        root = scihub[random.randint(0, 2)]

        wd.get(root + doi)
        time.sleep(1)

        try:
            b = wd.find_element_by_xpath('//*[@id="buttons"]/button')
            b.click()
            time.sleep(20)  # Wait for the download to start
        except:
            print('Access failed. DOI = ' + doi)
            wd.quit()
            return False

        wd.quit()
        return True

    def rename_file(download_path, title, doi):
        time.sleep(5)  # Wait a bit for the download to finish

        # Wait until the download is finished
        temp_file = max([download_path + f for f in os.listdir(download_path)], key=os.path.getctime)
        while '.crdownload' in temp_file:
            time.sleep(1)
            temp_file = max([download_path + f for f in os.listdir(download_path)], key=os.path.getctime)

        clean_title = clean_filename(title)
        clean_doi = clean_filename(doi).replace('/', '_')
        new_file_name = f"{clean_title}_{clean_doi}.pdf"
        os.rename(temp_file, download_path + new_file_name)

    def article_get(titles, dois, i):
        script_directory = os.path.dirname(os.path.abspath(__file__))

        download_path = os.path.join(script_directory, 'download_literatures')

        if not os.path.exists(download_path):
            os.makedirs(download_path)

        title = titles[i]
        doi = dois[i]

        # Visit Sci-Hub
        if doi == 'nan' or title == 'nan':
            print('Missing information. Index = ' + str(i))
        else:
            if scihub_get(doi, download_path):
                rename_file(download_path, title, doi)

    for i in range(len(dois)):
        article_get(titles, dois, i)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from setuptools import setup, find_packages

# open('README.md').read() # ChatGPT4用的这个，孙朋川自己改了下，为了预防没有自动关闭README.md的问题。
with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='ptol',
    version='0.1.1',
    python_requires='>=3.8',
    author='Huilong Chen',
    author_email='chenhuilong131@163.com',
    description='A Pipeline for Obtaining Relevant Literature Based on Given Keywords',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="MIT License",
    url="https://github.com/ChenHuilong1223/Pipeline_to_literature",
    packages=find_packages(),
    install_requires=[
        'pandas',
        'selenium',
        'nltk',
        'inflect',
        'PyPDF2',
        'pdfplumber',
        'PyMuPDF', # 注意：fitz 现在是 PyMuPDF 的一部分
        'pdfminer.six'
    ],
    entry_points={
        'console_scripts': [
            'ptol=ptol.main:main',
        ],
    },
    classifiers=[
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],

)
